@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- *  gnome-print-pdf.c: A PDF driver for GnomePrint
+ *  gnome-print-pdf.c: the PDF backend
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public License
@@ -26,10 +26,7 @@
  *    [1] Portable Document Format Referece Manual, Version 1.3 (March 11, 1999)
  */
 
-#define __GNOME_PRINT_PDF_C__
-
-#include "config.h"
-
+#include <config.h>
 #include <math.h>
 #include <string.h>
 #include <time.h>
@@ -42,14 +39,14 @@
 #include <libgnomeprint/gp-gc-private.h>
 #include <libgnomeprint/gnome-print-transport.h>
 #include <libgnomeprint/gnome-font-private.h>
-#include <libgnomeprint/gnome-print-encode-private.h>
+#include <libgnomeprint/gnome-print-encode.h>
 #include <libgnomeprint/gnome-pgl-private.h>
 #include <libgnomeprint/gnome-print-pdf-private.h>
-
-#include "gnome-print-pdf.h"
+#include <libgnomeprint/gnome-print-pdf.h>
 
 #define GNOME_PRINT_PDF_INITIAL_BUFFER_SIZE 1024
 
+typedef struct _GnomePrintPdfClass GnomePrintPdfClass;
 typedef struct _GnomePrintPdfPage   GnomePrintPdfPage;
 typedef struct _GnomePrintPdfObject GnomePrintPdfObject;
 
@@ -182,6 +179,7 @@ static gchar *gnome_print_pdf_id_new (GnomePrintPdf *pdf);
 
 static gint  gnome_print_pdf_graphic_mode_set (GnomePrintPdf *pdf, PdfGraphicMode mode);
 static gint  gnome_print_pdf_page_fprintf (GnomePrintPdf *pdf, const char *format, ...);
+static gint  gnome_print_pdf_page_print_double (GnomePrintPdf *pdf, const gchar *format, gdouble x);
 
 static GnomePrintContextClass *parent_class;
 
@@ -388,9 +386,9 @@ gnome_print_pdf_clip (GnomePrintContext *ctx, const ArtBpath *bpath, ArtWindRule
 	gnome_print_pdf_print_bpath (pdf, bpath);
 
 	if (rule == ART_WIND_RULE_NONZERO) {
-		gnome_print_pdf_page_fprintf (pdf, "W" EOL);
+		gnome_print_pdf_page_fprintf (pdf, "W n" EOL);
 	} else {
-		gnome_print_pdf_page_fprintf (pdf, "W*" EOL);
+		gnome_print_pdf_page_fprintf (pdf, "W* n" EOL);
 	}
 
 	return GNOME_PRINT_OK;
@@ -519,8 +517,24 @@ gnome_print_pdf_image (GnomePrintContext *pc, const gdouble *ctm, const guchar *
 	else
 		page->used_color_images = TRUE;
 	gnome_print_pdf_page_fprintf (pdf, "q" EOL);
-	gnome_print_pdf_page_fprintf (pdf, "%g %g %g %g %g %g cm" EOL,
-				      ctm[0], ctm[1], ctm[2], ctm[3], ctm[4], ctm[5]);
+	gnome_print_pdf_page_print_double (pdf, "%g" EOL,
+				      ctm[0]);
+	gnome_print_pdf_page_fprintf (pdf, " ");
+	gnome_print_pdf_page_print_double (pdf, "%g" EOL,
+				      ctm[1]);
+	gnome_print_pdf_page_fprintf (pdf, " ");
+	gnome_print_pdf_page_print_double (pdf, "%g" EOL,
+				      ctm[2]);
+	gnome_print_pdf_page_fprintf (pdf, " ");
+	gnome_print_pdf_page_print_double (pdf, "%g" EOL,
+				      ctm[3]);
+	gnome_print_pdf_page_fprintf (pdf, " ");
+	gnome_print_pdf_page_print_double (pdf, "%g" EOL,
+				      ctm[4]);
+	gnome_print_pdf_page_fprintf (pdf, " ");
+	gnome_print_pdf_page_print_double (pdf, "%g" EOL,
+				      ctm[5]);
+	gnome_print_pdf_page_fprintf (pdf, " cm" EOL);
 	gnome_print_pdf_page_fprintf (pdf, "0 0 m" EOL);
 	gnome_print_pdf_page_fprintf (pdf, "/Im%d Do" EOL, object_num);
 	gnome_print_pdf_page_fprintf (pdf, "Q" EOL);
@@ -621,9 +635,15 @@ gnome_print_pdf_glyphlist (GnomePrintContext *pc, const gdouble *ctm, GnomeGlyph
 		ret = gnome_print_pdf_page_fprintf (pdf, "[");
 		g_return_val_if_fail (ret >= 0, ret);
 		for (i = ps->start + 1; i < ps->start + ps->length; i++) {
-			ret = gnome_print_pdf_page_fprintf (pdf, "%g %g ",
-							    pgl->glyphs[i].x - pgl->glyphs[i-1].x,
-							    pgl->glyphs[i].y - pgl->glyphs[i-1].y);
+			ret = gnome_print_pdf_page_print_double (pdf, "%g",
+				     pgl->glyphs[i].x - pgl->glyphs[i-1].x);
+			g_return_val_if_fail (ret >= 0, ret);
+			ret = gnome_print_pdf_page_fprintf (pdf, " ");
+			g_return_val_if_fail (ret >= 0, ret);
+			ret = gnome_print_pdf_page_print_double (pdf, "%g",
+				     pgl->glyphs[i].y - pgl->glyphs[i-1].y);
+			g_return_val_if_fail (ret >= 0, ret);
+			ret = gnome_print_pdf_page_fprintf (pdf, " ");
 			g_return_val_if_fail (ret >= 0, ret);
 		}
 		ret = gnome_print_pdf_page_fprintf (pdf, "0 0] ");
@@ -942,13 +962,13 @@ gnome_print_pdf_write_pages (GnomePrintPdf *pdf, gint object_number_catalog)
 		nth_page = (GnomePrintPdfPage *) list->data;
 		ret += gnome_print_pdf_object_start (pdf, nth_page->object_number_page, FALSE);
 		gnome_print_pdf_fprintf (pdf,
-						"/Type /Page" EOL
-						"/Parent %d 0 R" EOL
-						"/Resources %d 0 R" EOL
-						"/Contents %d 0 R" EOL,
-						object_number_pages,
-						nth_page->object_number_resources,
-						nth_page->object_number_contents);
+					 "/Type /Page" EOL
+					 "/Parent %d 0 R" EOL
+					 "/Resources %d 0 R" EOL
+					 "/Contents %d 0 R" EOL,
+					 object_number_pages,
+					 nth_page->object_number_resources,
+					 nth_page->object_number_contents);
 		ret += gnome_print_pdf_object_end   (pdf, nth_page->object_number_page, FALSE);
 		list = list->next;
 	}
@@ -1102,6 +1122,8 @@ gnome_print_pdf_close_write_last_objects (GnomePrintPdf *pdf)
 	for (; list != NULL; list = list->next) {
 		GnomePrintPdfObject *nth_object;
 		nth_object = (GnomePrintPdfObject *) list->data;
+		if (nth_object->offset < 1) 
+			g_warning ("Object with offset Zero while creating pdf file");
 		gnome_print_pdf_fprintf (pdf,
 					 "%010i %05i n",
 					 nth_object->offset,
@@ -1188,12 +1210,15 @@ gnome_print_pdf_set_line (GnomePrintPdf *pdf)
 
 	if (gp_gc_get_line_flag (ctx->gc) == GP_GC_FLAG_CLEAR)
 		return GNOME_PRINT_OK;
-
-	gnome_print_pdf_page_fprintf (pdf, "%g w %d J %d j %g M" EOL,
-				      gp_gc_get_linewidth (ctx->gc),
+	gnome_print_pdf_page_print_double (pdf, "%g", 
+					   gp_gc_get_linewidth (ctx->gc));
+	gnome_print_pdf_page_fprintf (pdf, "w %d J %d j ",
 				      gp_gc_get_linecap (ctx->gc),
 				      gp_gc_get_linejoin (ctx->gc),
 				      gp_gc_get_miterlimit (ctx->gc));
+	gnome_print_pdf_page_print_double (pdf, "%g", 
+					   gp_gc_get_miterlimit (ctx->gc));
+	gnome_print_pdf_page_fprintf (pdf, " M" EOL);
 	gp_gc_set_line_flag (ctx->gc, GP_GC_FLAG_CLEAR);
 
 	return GNOME_PRINT_OK;
@@ -1213,9 +1238,14 @@ gnome_print_pdf_set_dash (GnomePrintPdf *pdf)
 
 	dash = gp_gc_get_dash (ctx->gc);
 	gnome_print_pdf_page_fprintf (pdf, "[");
-	for (i = 0; i < dash->n_dash; i++)
-		gnome_print_pdf_page_fprintf (pdf, " %g", dash->dash[i]);
-	gnome_print_pdf_page_fprintf (pdf, "]%g d" EOL, dash->n_dash > 0 ? dash->offset : 0.0);
+	for (i = 0; i < dash->n_dash; i++) {
+		gnome_print_pdf_page_fprintf (pdf, " ");
+		gnome_print_pdf_page_print_double (pdf, "%g", dash->dash[i]);
+	}
+	gnome_print_pdf_page_fprintf (pdf, "]");
+	gnome_print_pdf_page_print_double (pdf, "%g", 
+				 dash->n_dash > 0 ? dash->offset : 0.0);
+	gnome_print_pdf_page_fprintf (pdf, " d" EOL);
 	gp_gc_set_dash_flag (ctx->gc, GP_GC_FLAG_CLEAR);
 
 	return GNOME_PRINT_OK;
@@ -1295,7 +1325,8 @@ gnome_print_pdf_font_print_widths (GnomePrintPdf *pdf, GnomePrintPdfFont *font)
 			gnome_print_pdf_fprintf (pdf, EOL);
 			col = 0;
 		}
-		col += gnome_print_pdf_fprintf (pdf, "%g ", point.x);
+		col += gnome_print_pdf_print_double (pdf, "%g", point.x);
+		col += gnome_print_pdf_fprintf (pdf, " ");
 		i++;
 	}
 	
@@ -1408,9 +1439,9 @@ my_fix_drect (ArtDRect *rect)
 #undef TOO_SMALL
 }
 
-static void
+static GnomePrintReturnCode
 gnome_print_pdf_font_print_descriptor (GnomePrintPdf *pdf, GnomePrintPdfFont *font,
-				       gint object_number_descriptor)
+				       gint *object_number_descriptor_ret)
 {
 	const ArtDRect *rect;
 	ArtDRect new_rect;
@@ -1418,12 +1449,25 @@ gnome_print_pdf_font_print_descriptor (GnomePrintPdf *pdf, GnomePrintPdfFont *fo
 	gint ascent, descent, flags, stemv, stemh;
 	gint italic_angle, capheight, xheight;
 	gdouble val;
-	gint fontfile_object_number = gnome_print_pdf_object_new (pdf);
+	gint fontfile_object_number;
+	gint object_number_descriptor;
 	gint embed_result;
+	const gchar *file_name;
 
 	face = font->face;
 
-	/* Get the values */
+	file_name = font->face->entry->file;
+	if (font->is_type_1) {
+		embed_result = gnome_print_pdf_t1_embed (pdf, file_name, &fontfile_object_number);
+	} else {
+		embed_result = gnome_print_pdf_tt_embed (pdf, file_name, &fontfile_object_number);
+	}
+
+	if (embed_result != GNOME_PRINT_OK) {
+		g_print ("Could not embed font %s\n", gnome_font_face_get_ps_name (font->face));		
+		return GNOME_PRINT_ERROR_UNKNOWN;
+	}
+
 	ascent  = (gint) gnome_font_face_get_ascender (face);
 	descent = (gint) gnome_font_face_get_descender (face);
 	flags = gnome_font_face_get_pdf_flags (face);
@@ -1442,16 +1486,9 @@ gnome_print_pdf_font_print_descriptor (GnomePrintPdf *pdf, GnomePrintPdfFont *fo
 	art_drect_copy (&new_rect, rect);
 	my_fix_drect (&new_rect);
 
-	/* Embed the font first, then print the descriptor so that if the font
-	 * embeding fails, we don't add the /FontFile tag (Chema)
-	 */
-	if (font->is_type_1)
-		embed_result = gnome_print_pdf_t1_embed (pdf,
-							  font, fontfile_object_number);
-	else
-		embed_result = gnome_print_pdf_tt_embed (pdf,
-							  font, fontfile_object_number);
-
+	
+	object_number_descriptor = gnome_print_pdf_object_new (pdf);
+	*object_number_descriptor_ret = object_number_descriptor;
 	/* Print Descriptor */
 	gnome_print_pdf_object_start (pdf, object_number_descriptor, FALSE);
 	gnome_print_pdf_fprintf (pdf,
@@ -1460,14 +1497,22 @@ gnome_print_pdf_font_print_descriptor (GnomePrintPdf *pdf, GnomePrintPdfFont *fo
 				 "/CapHeight %d" EOL
 				 "/Descent %d" EOL
 				 "/Flags %d" EOL
-				 "/FontBBox [%g %g %g %g]" EOL
+				 "/FontBBox [",
+				 ascent, capheight,
+				 descent * -1, flags);
+	gnome_print_pdf_print_double (pdf, "%g", new_rect.x0);
+	gnome_print_pdf_fprintf (pdf, " ");
+	gnome_print_pdf_print_double (pdf, "%g", new_rect.y0);
+	gnome_print_pdf_fprintf (pdf, " ");
+	gnome_print_pdf_print_double (pdf, "%g", new_rect.x1);
+	gnome_print_pdf_fprintf (pdf, " ");
+	gnome_print_pdf_print_double (pdf, "%g", new_rect.y1);
+	gnome_print_pdf_fprintf (pdf,
+				 "]" EOL
 				 "/FontName /%s" EOL
 				 "/ItalicAngle %d" EOL
 				 "/StemV %d" EOL
 				 "/XHeight %d" EOL,
-				 ascent, capheight,
-				 descent * -1, flags,
-				 new_rect.x0, new_rect.y0, new_rect.x1, new_rect.y1,
 				 gnome_font_face_get_ps_name (font->face),
 				 italic_angle, stemv, xheight);
 
@@ -1479,6 +1524,8 @@ gnome_print_pdf_font_print_descriptor (GnomePrintPdf *pdf, GnomePrintPdfFont *fo
 					 fontfile_object_number	);
 				 
 	gnome_print_pdf_object_end   (pdf, object_number_descriptor, FALSE);
+
+	return GNOME_PRINT_OK;
 }
 		
 static gint
@@ -1490,6 +1537,7 @@ gnome_print_pdf_set_font_real (GnomePrintPdf *pdf, const GnomeFont *gnome_font)
 	GList *needle;
 	gint object_number_descriptor = 0;
 	gint i;
+	gboolean plan_b = FALSE; /* TRUE if we could not embed the font */
 
 	face = gnome_font->face;
 	
@@ -1509,7 +1557,7 @@ gnome_print_pdf_set_font_real (GnomePrintPdf *pdf, const GnomeFont *gnome_font)
 	font = g_new (GnomePrintPdfFont, 1);
 	font->face = gnome_font_get_face (gnome_font);
 	g_object_ref (font->face);
-	font->pso  = gnome_font_face_pso_new (font->face, NULL);
+	font->pso  = gnome_font_face_pso_new (font->face, NULL, 0);
 	font->is_basic_14 = FALSE;
 	font->nglyphs = gnome_font_face_get_num_glyphs (face);
 	font->glyph_to_code = g_hash_table_new (NULL, NULL);
@@ -1532,8 +1580,7 @@ gnome_print_pdf_set_font_real (GnomePrintPdf *pdf, const GnomeFont *gnome_font)
 	font->object_number = gnome_print_pdf_object_new (pdf);
 
 	/* Make sure this backend can handle this font type */
-	if (face->entry->type == GP_FONT_ENTRY_TYPE1 ||
-	    face->entry->type == GP_FONT_ENTRY_TYPE1_ALIAS)
+	if (face->entry->type == GP_FONT_ENTRY_TYPE1)
 		font->is_type_1 = TRUE;
 	else if (face->entry->type == GP_FONT_ENTRY_TRUETYPE)
 		font->is_type_1 = FALSE;
@@ -1545,6 +1592,18 @@ gnome_print_pdf_set_font_real (GnomePrintPdf *pdf, const GnomeFont *gnome_font)
 	/* Add it to the list of fonts of this job */
 	pdf->fonts = g_list_prepend (pdf->fonts, font);
 
+	if (!font->is_basic_14) {
+		GnomePrintReturnCode retval;
+		retval =  gnome_print_pdf_font_print_descriptor (pdf, font,
+								 &object_number_descriptor);
+		if (retval != GNOME_PRINT_OK) {
+			g_warning ("Could not embed font %s, using Times-Roman instead.",
+				   gnome_font_face_get_ps_name (font->face));
+			plan_b = TRUE;
+			font->is_type_1 = TRUE; /* It isn't, but the substitute is */
+		}
+	}
+
 	/* Write the object data */
 	gnome_print_pdf_object_start (pdf, font->object_number, FALSE);
 	gnome_print_pdf_fprintf  (pdf,
@@ -1553,30 +1612,28 @@ gnome_print_pdf_set_font_real (GnomePrintPdf *pdf, const GnomeFont *gnome_font)
 				  "/BaseFont /%s" EOL
 				  "/Name /F%i" EOL,
 				  font->is_type_1 ? "Type1" : "TrueType",
-				  gnome_font_face_get_ps_name (font->face),
+				  plan_b ? (const guchar *) "Times-Roman" : gnome_font_face_get_ps_name (font->face),
 				  font->object_number);
 
 	if (!font->is_basic_14) {
-		object_number_descriptor = gnome_print_pdf_object_new (pdf);
 		gnome_print_pdf_fprintf  (pdf,
 					  "/FirstChar %d" EOL
 					  "/LastChar %d 0 R" EOL
 					  "/Encoding %i 0 R" EOL
-					  "/Widths %d 0 R" EOL
-					  "/FontDescriptor %d 0 R" EOL,
+					  "/Widths %d 0 R" EOL,
 					  0,
 					  font->object_number_lastchar,
 					  font->object_number_encoding,
-					  font->object_number_widths,
+					  font->object_number_widths);
+	}
+	if (!plan_b) {
+		gnome_print_pdf_fprintf (pdf,
+					 "/FontDescriptor %d 0 R" EOL,
 					  object_number_descriptor);
 	}
 	
 	gnome_print_pdf_object_end   (pdf, font->object_number, FALSE);
 
-	if (!font->is_basic_14) {
-		gnome_print_pdf_font_print_descriptor (pdf, font, object_number_descriptor);
-	}
-
 gnome_print_pdf_set_font_real_page_stuff:
 	/* Is the font alraedy set? If it is just return */
 	if (pdf->selected_font == font)
@@ -1622,8 +1679,6 @@ gnome_print_pdf_print_bpath (GnomePrintPdf *pdf, const ArtBpath *bpath)
 {
 	gboolean started, closed;
 	
-	gnome_print_pdf_page_fprintf (pdf, "n" EOL);
-
 	started = FALSE;
 	closed = FALSE;
 	while (bpath->code != ART_END) {
@@ -1633,23 +1688,40 @@ gnome_print_pdf_print_bpath (GnomePrintPdf *pdf, const ArtBpath *bpath)
 				gnome_print_pdf_page_fprintf (pdf, "h" EOL);
 			closed = FALSE;
 			started = FALSE;
-			gnome_print_pdf_page_fprintf (pdf, "%g %g m" EOL, bpath->x3, bpath->y3);
+			gnome_print_pdf_page_print_double (pdf, "%g" EOL, bpath->x3);
+			gnome_print_pdf_page_fprintf (pdf, " ");
+			gnome_print_pdf_page_print_double (pdf, "%g" EOL, bpath->y3);
+			gnome_print_pdf_page_fprintf (pdf, " m" EOL);
 			break;
 		case ART_MOVETO:
 			if (started && closed)
 				gnome_print_pdf_page_fprintf (pdf, "h" EOL);
 			closed = TRUE;
 			started = TRUE;
-			gnome_print_pdf_page_fprintf (pdf, "%g %g m" EOL, bpath->x3, bpath->y3);
+			gnome_print_pdf_page_print_double (pdf, "%g" EOL, bpath->x3);
+			gnome_print_pdf_page_fprintf (pdf, " ");
+			gnome_print_pdf_page_print_double (pdf, "%g" EOL, bpath->y3);
+			gnome_print_pdf_page_fprintf (pdf, " m" EOL);
 			break;
 		case ART_LINETO:
-			gnome_print_pdf_page_fprintf (pdf, "%g %g l" EOL, bpath->x3, bpath->y3);
+			gnome_print_pdf_page_print_double (pdf, "%g" EOL, bpath->x3);
+			gnome_print_pdf_page_fprintf (pdf, " ");
+			gnome_print_pdf_page_print_double (pdf, "%g" EOL, bpath->y3);
+			gnome_print_pdf_page_fprintf (pdf, " l" EOL);
 			break;
 		case ART_CURVETO:
-			gnome_print_pdf_page_fprintf (pdf, "%g %g %g %g %g %g c" EOL,
-						      bpath->x1, bpath->y1,
-						      bpath->x2, bpath->y2,
-						      bpath->x3, bpath->y3);
+			gnome_print_pdf_page_print_double (pdf, "%g" EOL, bpath->x1);
+			gnome_print_pdf_page_fprintf (pdf, " ");
+			gnome_print_pdf_page_print_double (pdf, "%g" EOL, bpath->y1);
+			gnome_print_pdf_page_fprintf (pdf, " ");
+			gnome_print_pdf_page_print_double (pdf, "%g" EOL, bpath->x2);
+			gnome_print_pdf_page_fprintf (pdf, " ");
+			gnome_print_pdf_page_print_double (pdf, "%g" EOL, bpath->y2);
+			gnome_print_pdf_page_fprintf (pdf, " ");
+			gnome_print_pdf_page_print_double (pdf, "%g" EOL, bpath->x3);
+			gnome_print_pdf_page_fprintf (pdf, " ");
+			gnome_print_pdf_page_print_double (pdf, "%g" EOL, bpath->y3);
+			gnome_print_pdf_page_fprintf (pdf, " c" EOL);
 			break;
 		default:
 			g_warning ("Path structure is corrupted");
@@ -1724,16 +1796,11 @@ gnome_print_pdf_fprintf (GnomePrintPdf *pdf, const char *format, ...)
 {
  	va_list arguments;
  	gchar *text;
-	gchar *oldlocale;
 	gint len;
 
-	oldlocale = g_strdup (setlocale (LC_NUMERIC, NULL));
-	setlocale (LC_NUMERIC, "C");
  	va_start (arguments, format);
  	text = g_strdup_vprintf (format, arguments);
  	va_end (arguments);
-	setlocale (LC_NUMERIC, oldlocale);
-	g_free (oldlocale);
 
 	len = strlen (text);
 	gnome_print_transport_write (GNOME_PRINT_CONTEXT (pdf)->transport, text, len);
@@ -1743,6 +1810,24 @@ gnome_print_pdf_fprintf (GnomePrintPdf *pdf, const char *format, ...)
 	return len;
 }
 
+gint 
+gnome_print_pdf_print_double (GnomePrintPdf *pdf, const gchar *format, gdouble x)
+{
+ 	gchar *text;
+	gint len;
+
+	text = g_new (gchar, G_ASCII_DTOSTR_BUF_SIZE);
+	g_ascii_formatd (text, G_ASCII_DTOSTR_BUF_SIZE, format, x);
+
+	len = strlen (text);
+	gnome_print_transport_write (GNOME_PRINT_CONTEXT (pdf)->transport, text, len);
+	pdf->offset += len;
+	g_free (text);
+
+	return len;
+}
+
+
 gint
 gnome_print_pdf_print_sized (GnomePrintPdf *pdf, const char *content, gint len)
 {
@@ -1753,25 +1838,12 @@ gnome_print_pdf_print_sized (GnomePrintPdf *pdf, const char *content, gint len)
 }
 
 static gint
-gnome_print_pdf_page_fprintf (GnomePrintPdf *pdf, const char *format, ...)
+gnome_print_pdf_page_write (GnomePrintPdf *pdf, gchar *text)
 {
-	gint grow_size = GNOME_PRINT_PDF_INITIAL_BUFFER_SIZE;
-	GnomePrintPdfPage *page;
- 	va_list arguments;
- 	gchar *text;
-	gchar *oldlocale;
 	gint len;
+	GnomePrintPdfPage *page;
+	gint grow_size = GNOME_PRINT_PDF_INITIAL_BUFFER_SIZE;
 
-	/* Compose the text */
-	oldlocale = g_strdup (setlocale (LC_NUMERIC, NULL));
-	setlocale (LC_NUMERIC, "C");
- 	va_start (arguments, format);
- 	text = g_strdup_vprintf (format, arguments);
- 	va_end (arguments);
-	setlocale (LC_NUMERIC, oldlocale);
-	g_free (oldlocale);
-	
-	/* Write it */
 	len = strlen (text);
 	page = pdf->pages->data;
 	if ((pdf->stream_used + len + 1) > pdf->stream_allocated) {
@@ -1785,11 +1857,50 @@ gnome_print_pdf_page_fprintf (GnomePrintPdf *pdf, const char *format, ...)
 	memcpy (pdf->stream + pdf->stream_used, text, len * sizeof (gchar));
 	pdf->stream_used += len;
 	pdf->stream [pdf->stream_used] = 0;
+
+	return len;
+}
+
+
+/* Note "format" should be locale independent, so it should not use %g */
+/* and friends */
+static gint
+gnome_print_pdf_page_fprintf (GnomePrintPdf *pdf, const char *format, ...)
+{
+ 	va_list arguments;
+	gint len;
+ 	gchar *text;
+
+	/* Compose the text */
+ 	va_start (arguments, format);
+ 	text = g_strdup_vprintf (format, arguments);
+ 	va_end (arguments);
+	
+	/* Write it */
+	len = gnome_print_pdf_page_write (pdf, text);
+
 	g_free (text);
 	
 	return len;
 }
 
+/* Allowed conversion specifiers are 'e', 'E', 'f', 'F', 'g' and 'G'. */
+static gint   
+gnome_print_pdf_page_print_double (GnomePrintPdf *pdf, 
+				   const gchar *format, gdouble x)
+{
+ 	gchar *text;
+	gint len;
+
+	text = g_new (gchar, G_ASCII_DTOSTR_BUF_SIZE);
+	g_ascii_formatd (text, G_ASCII_DTOSTR_BUF_SIZE, format, x);
+
+	len = gnome_print_pdf_page_write (pdf, text);
+	g_free (text);
+
+	return len;
+}
+
 static gint
 gnome_print_pdf_graphic_mode_set (GnomePrintPdf *pdf, PdfGraphicMode mode)
 {
