@@ -38,11 +38,20 @@
 #include <cups/cups.h>
 #include <cups/ppd.h>
 
+#include "libgnomeprint/gnome-print-i18n.h"
+
 /* We are protecting the locale due to a known bug in the libcups library. */
 #define LOCALEPROTECTION(c) { char *OLDLOCALE;\
                               OLDLOCALE = g_strdup(setlocale(LC_ALL, NULL)); \
                               c; \
-                              setlocale (LC_ALL, OLDLOCALE);\
+                              if (strcmp (OLDLOCALE, setlocale (LC_ALL, NULL))) {\
+                                       static gboolean LOCALEPROTECTIONWARNING = FALSE;\
+                                       if (!LOCALEPROTECTIONWARNING) {\
+                                                LOCALEPROTECTIONWARNING = TRUE;\
+                                                g_warning ("Working around locale bug in cups.");\
+                                       }\
+                                       setlocale (LC_ALL, OLDLOCALE);\
+			      }\
                               g_free (OLDLOCALE);\
                             }
  
@@ -77,7 +86,11 @@ xmlChar *model_xml_template =
 "      </Option>"
 "      <Option Id=\"Job\">"
 "        <Option Id=\"NumCopies\" Type=\"String\" Default=\"1\"/>"
+"        <Option Id=\"NonCollatedCopiesHW\" Type=\"String\" Default=\"true\"/>"
+"        <Option Id=\"CollatedCopiesHW\" Type=\"String\" Default=\"false\"/>"
 "        <Option Id=\"Collate\" Type=\"String\" Default=\"false\"/>"
+"        <Option Id=\"Duplex\" Type=\"String\" Default=\"true\"/>"
+"        <Option Id=\"Tumble\" Type=\"String\" Default=\"false\"/>"
 "        <Option Id=\"PrintToFile\" Type=\"String\" Default=\"false\" Locked=\"true\"/>"
 "        <Option Id=\"FileName\" Type=\"String\" Default=\"output.ps\"/>"
 "      </Option>"
@@ -231,7 +244,34 @@ load_paper_sources (ppd_file_t *ppd, GPANode *parent)
 	for (i = 0; i < option->num_choices; i++)
 		gpa_option_item_new (node,
 				     option->choices[i].choice,
-				     option->choices[i].choice);
+				     option->choices[i].text);
+}
+
+/* load_cups_hold_types */
+/* we could either make an ipp request for job-hold-until-supported */
+/* or list here the cups supported values */
+/* Sicne these values are in fact not printer specific but cups specific, */
+/* We'll just list them here. */
+static void
+load_cups_hold_types (GPANode *parent)
+{
+	GPANode *node;
+	
+	node = gpa_option_list_new (parent, "Hold", "no-hold");
+
+	if (!node)
+		return;
+	
+	gpa_option_item_new (node,"weekend",_("on the weekend"));
+	gpa_option_item_new (node,"third-shift",
+			     _("between midnight and 8 a.m."));
+/* 	gpa_option_item_new (node,"night",_("between 6 p.m. and 6 a.m.")); */
+	gpa_option_item_new (node,"evening",_("between 6 p.m. and 6 a.m."));
+	gpa_option_item_new (node,"second-shift",
+			     _("between 4 p.m. and midnight"));
+	gpa_option_item_new (node,"day-time",_("between 6 a.m. and 6 p.m."));
+	gpa_option_item_new (node,"indefinite",_("when manually released"));
+	gpa_option_item_new (node,"no-hold",_("immediately"));
 }
 
 static GPAModel *
@@ -240,6 +280,7 @@ get_model (const gchar *printer, ppd_file_t *ppd)
 	GPANode *media;
 	GPANode *model;
 	GPANode *output;
+	GPANode *job;
 	char *xml;
 	char *id;
 
@@ -258,12 +299,15 @@ get_model (const gchar *printer, ppd_file_t *ppd)
 
 	output = gpa_node_lookup (model, "Options.Output");
 	media  = gpa_node_lookup (model, "Options.Output.Media");
+	job  = gpa_node_lookup (model, "Options.Output.Job");
 
 	load_paper_sizes   (ppd, media);
 	load_paper_sources (ppd, output);
+	load_cups_hold_types (job);
 
 	gpa_node_unref (output);
 	gpa_node_unref (media);
+	gpa_node_unref (job);
 
 	g_free (id);
 	
